@@ -1,63 +1,60 @@
-####**********************************************************************
-####**********************************************************************
-####
-####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.4.1 (_PROJECT_BUILD_ID_)
-####
-####  Copyright 2016, University of Miami
-####
-####  This program is free software; you can redistribute it and/or
-####  modify it under the terms of the GNU General Public License
-####  as published by the Free Software Foundation; either version 3
-####  of the License, or (at your option) any later version.
-####
-####  This program is distributed in the hope that it will be useful,
-####  but WITHOUT ANY WARRANTY; without even the implied warranty of
-####  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-####  GNU General Public License for more details.
-####
-####  You should have received a copy of the GNU General Public
-####  License along with this program; if not, write to the Free
-####  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-####  Boston, MA  02110-1301, USA.
-####
-####  ----------------------------------------------------------------
-####  Project Partially Funded By: 
-####  ----------------------------------------------------------------
-####  Dr. Ishwaran's work was funded in part by DMS grant 1148991 from the
-####  National Science Foundation and grant R01 CA163739 from the National
-####  Cancer Institute.
-####
-####  Dr. Kogalur's work was funded in part by grant R01 CA163739 from the 
-####  National Cancer Institute.
-####  ----------------------------------------------------------------
-####  Written by:
-####  ----------------------------------------------------------------
-####    Hemant Ishwaran, Ph.D.
-####    Director of Statistical Methodology
-####    Professor, Division of Biostatistics
-####    Clinical Research Building, Room 1058
-####    1120 NW 14th Street
-####    University of Miami, Miami FL 33136
-####
-####    email:  hemant.ishwaran@gmail.com
-####    URL:    http://web.ccs.miami.edu/~hishwaran
-####    --------------------------------------------------------------
-####    Udaya B. Kogalur, Ph.D.
-####    Adjunct Staff
-####    Department of Quantitative Health Sciences
-####    Cleveland Clinic Foundation
-####    
-####    Kogalur & Company, Inc.
-####    5425 Nestleway Drive, Suite L1
-####    Clemmons, NC 27012
-####
-####    email:  ubk@kogalur.com
-####    URL:    http://www.kogalur.com
-####    --------------------------------------------------------------
-####
-####**********************************************************************
-####**********************************************************************
+##  **********************************************************************
+##  **********************************************************************
+##  
+##    RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
+##  
+##    This program is free software; you can redistribute it and/or
+##    modify it under the terms of the GNU General Public License
+##    as published by the Free Software Foundation; either version 3
+##    of the License, or (at your option) any later version.
+##  
+##    This program is distributed in the hope that it will be useful,
+##    but WITHOUT ANY WARRANTY; without even the implied warranty of
+##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+##    GNU General Public License for more details.
+##  
+##    You should have received a copy of the GNU General Public
+##    License along with this program; if not, write to the Free
+##    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+##    Boston, MA  02110-1301, USA.
+##  
+##    ----------------------------------------------------------------
+##    Project Partially Funded By: 
+##    ----------------------------------------------------------------
+##    Dr. Ishwaran's work was funded in part by DMS grant 1148991 from the
+##    National Science Foundation and grant R01 CA163739 from the National
+##    Cancer Institute.
+##  
+##    Dr. Kogalur's work was funded in part by grant R01 CA163739 from the 
+##    National Cancer Institute.
+##    ----------------------------------------------------------------
+##    Written by:
+##    ----------------------------------------------------------------
+##      Hemant Ishwaran, Ph.D.
+##      Director of Statistical Methodology
+##      Professor, Division of Biostatistics
+##      Clinical Research Building, Room 1058
+##      1120 NW 14th Street
+##      University of Miami, Miami FL 33136
+##  
+##      email:  hemant.ishwaran@gmail.com
+##      URL:    http://web.ccs.miami.edu/~hishwaran
+##      --------------------------------------------------------------
+##      Udaya B. Kogalur, Ph.D.
+##      Adjunct Staff
+##      Department of Quantitative Health Sciences
+##      Cleveland Clinic Foundation
+##      
+##      Kogalur & Company, Inc.
+##      5425 Nestleway Drive, Suite L1
+##      Clemmons, NC 27012
+##  
+##      email:  ubk@kogalur.com
+##      URL:    https://github.com/kogalur/randomForestSRC
+##      --------------------------------------------------------------
+##  
+##  **********************************************************************
+##  **********************************************************************
 
 
 adrop3d.last <- function(x, d, keepColNames = FALSE) {
@@ -368,7 +365,9 @@ get.coerced.survival.fmly <- function(fmly, event.type, splitrule = NULL) {
   if (grepl("surv", fmly)) {
     coerced.fmly <- "surv"
     if (!is.null(splitrule)) {
-      if ((length(event.type) > 1) & (splitrule != "logrankscore")) {
+        if ((length(event.type) > 1) &&
+            (splitrule != "l2.impute") &&
+            (splitrule != "logrankscore")) {
         coerced.fmly <- "surv-CR"
       }
     }
@@ -424,7 +423,7 @@ get.grow.event.info <- function(yvar, fmly, need.deaths = TRUE, ntime) {
     if (!all(floor(cens) == abs(cens), na.rm = TRUE)) {
       stop("for survival families censoring variable must be coded as a non-negative integer (perhaps the formula is set incorrectly?)")
     }
-    if (need.deaths & (all(na.omit(cens) == 0))) {
+    if (need.deaths && (all(na.omit(cens) == 0))) {
       stop("no deaths in data!")
     }
     if (!all(na.omit(time) >= 0)) {
@@ -484,7 +483,8 @@ get.grow.splitinfo <- function (formula.detail, splitrule, nsplit, event.type) {
                        "unsupv",               
                        "mv.mse",               
                        "mv.gini",              
-                       "custom")               
+                       "custom",               
+                       "l2.impute")            
   fmly <- formula.detail$family
   nsplit <- round(nsplit)
   if (nsplit < 0) {
