@@ -1,63 +1,60 @@
-####**********************************************************************
-####**********************************************************************
-####
-####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.4.1 (_PROJECT_BUILD_ID_)
-####
-####  Copyright 2016, University of Miami
-####
-####  This program is free software; you can redistribute it and/or
-####  modify it under the terms of the GNU General Public License
-####  as published by the Free Software Foundation; either version 3
-####  of the License, or (at your option) any later version.
-####
-####  This program is distributed in the hope that it will be useful,
-####  but WITHOUT ANY WARRANTY; without even the implied warranty of
-####  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-####  GNU General Public License for more details.
-####
-####  You should have received a copy of the GNU General Public
-####  License along with this program; if not, write to the Free
-####  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-####  Boston, MA  02110-1301, USA.
-####
-####  ----------------------------------------------------------------
-####  Project Partially Funded By: 
-####  ----------------------------------------------------------------
-####  Dr. Ishwaran's work was funded in part by DMS grant 1148991 from the
-####  National Science Foundation and grant R01 CA163739 from the National
-####  Cancer Institute.
-####
-####  Dr. Kogalur's work was funded in part by grant R01 CA163739 from the 
-####  National Cancer Institute.
-####  ----------------------------------------------------------------
-####  Written by:
-####  ----------------------------------------------------------------
-####    Hemant Ishwaran, Ph.D.
-####    Director of Statistical Methodology
-####    Professor, Division of Biostatistics
-####    Clinical Research Building, Room 1058
-####    1120 NW 14th Street
-####    University of Miami, Miami FL 33136
-####
-####    email:  hemant.ishwaran@gmail.com
-####    URL:    http://web.ccs.miami.edu/~hishwaran
-####    --------------------------------------------------------------
-####    Udaya B. Kogalur, Ph.D.
-####    Adjunct Staff
-####    Department of Quantitative Health Sciences
-####    Cleveland Clinic Foundation
-####    
-####    Kogalur & Company, Inc.
-####    5425 Nestleway Drive, Suite L1
-####    Clemmons, NC 27012
-####
-####    email:  ubk@kogalur.com
-####    URL:    http://www.kogalur.com
-####    --------------------------------------------------------------
-####
-####**********************************************************************
-####**********************************************************************
+##  **********************************************************************
+##  **********************************************************************
+##  
+##    RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
+##  
+##    This program is free software; you can redistribute it and/or
+##    modify it under the terms of the GNU General Public License
+##    as published by the Free Software Foundation; either version 3
+##    of the License, or (at your option) any later version.
+##  
+##    This program is distributed in the hope that it will be useful,
+##    but WITHOUT ANY WARRANTY; without even the implied warranty of
+##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+##    GNU General Public License for more details.
+##  
+##    You should have received a copy of the GNU General Public
+##    License along with this program; if not, write to the Free
+##    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+##    Boston, MA  02110-1301, USA.
+##  
+##    ----------------------------------------------------------------
+##    Project Partially Funded By: 
+##    ----------------------------------------------------------------
+##    Dr. Ishwaran's work was funded in part by DMS grant 1148991 from the
+##    National Science Foundation and grant R01 CA163739 from the National
+##    Cancer Institute.
+##  
+##    Dr. Kogalur's work was funded in part by grant R01 CA163739 from the 
+##    National Cancer Institute.
+##    ----------------------------------------------------------------
+##    Written by:
+##    ----------------------------------------------------------------
+##      Hemant Ishwaran, Ph.D.
+##      Director of Statistical Methodology
+##      Professor, Division of Biostatistics
+##      Clinical Research Building, Room 1058
+##      1120 NW 14th Street
+##      University of Miami, Miami FL 33136
+##  
+##      email:  hemant.ishwaran@gmail.com
+##      URL:    http://web.ccs.miami.edu/~hishwaran
+##      --------------------------------------------------------------
+##      Udaya B. Kogalur, Ph.D.
+##      Adjunct Staff
+##      Department of Quantitative Health Sciences
+##      Cleveland Clinic Foundation
+##      
+##      Kogalur & Company, Inc.
+##      5425 Nestleway Drive, Suite L1
+##      Clemmons, NC 27012
+##  
+##      email:  ubk@kogalur.com
+##      URL:    https://github.com/kogalur/randomForestSRC
+##      --------------------------------------------------------------
+##  
+##  **********************************************************************
+##  **********************************************************************
 
 
 get.bootstrap <- function (bootstrap) {
@@ -104,7 +101,7 @@ get.na.action <- function (na.action) {
     else if (na.action == "na.impute") {
       na.action <- 2^4
     }
-      else {
+    else {
         stop("Invalid choice for 'na.action' option:  ", na.action)
       }
   return (na.action)
@@ -236,7 +233,7 @@ get.perf.bits <- function (perf) {
       return (0)
     }
 }
-  get.proximity <- function (grow.equivalent, proximity) {
+get.proximity <- function (grow.equivalent, proximity) {
     if (!is.null(proximity)) {
       if (proximity == FALSE) {
         prox.bits <- 0
@@ -378,10 +375,10 @@ get.perf.bits <- function (perf) {
   get.var.used <- function (var.used) {
     if (!is.null(var.used)) {
       if (var.used == "all.trees") {
-        var.used <- 2^13 + 0
+        var.used <- 2^12
       }
         else if (var.used == "by.tree") {
-          var.used <- 2^13 + 2^12
+          var.used <- 2^13
         }
           else if (var.used == FALSE) {
             var.used <- 0
